#include "AdaptStationary.hpp"

// AMDiS includes
#include "AdaptInfo.hpp"
#include "Flag.hpp"
#include "Initfile.hpp"
#include "ProblemIterationInterface.hpp"

namespace AMDiS
{

  AdaptStationary::AdaptStationary(std::string name,
                                   ProblemIterationInterface& prob,
                                   AdaptInfo& adaptInfo)
    : AdaptBase(name, &prob, adaptInfo)
  {
    Parameters::get(name + "->info", info);
  }


  int AdaptStationary::adapt()
  {
    // initial iteration
    if (adaptInfo.getSpaceIteration() == -1)
    {
      problemIteration->beginIteration(adaptInfo);
      problemIteration->oneIteration(adaptInfo, NO_ADAPTION);
      problemIteration->endIteration(adaptInfo);
      adaptInfo.incSpaceIteration();
    }

    // adaption loop
    while (!adaptInfo.spaceToleranceReached() &&
           (adaptInfo.getSpaceIteration() < adaptInfo.getMaxSpaceIteration() ||
            adaptInfo.getMaxSpaceIteration() < 0) )
    {

      problemIteration->beginIteration(adaptInfo);
      Flag adapted = problemIteration->oneIteration(adaptInfo, FULL_ITERATION);
      problemIteration->endIteration(adaptInfo);

      if (adapted == Flag{0})
        break;

      adaptInfo.incSpaceIteration();
    }

    return 0;
  }

} // end namespace AMDiS
