#pragma once

#include <dune/functions/functionspacebases/interpolate.hh>
#include <dune/amdis/LinearAlgebra.hpp>

namespace AMDiS
{
  template <class WorldVector>
    template <class Matrix, class VectorX, class VectorB>
  void DirichletBC<WorldVector>::init(bool apply,
				      Matrix& matrix,
                                      VectorX& solution,
				      VectorB& rhs)
  {
    using Dune::Functions::interpolate;

    if (!initialized) {
      interpolate(matrix.getRowFeSpace(), dirichletNodes, predicate);
      initialized = true;
    }
  }


  template <class WorldVector>
    template <class Matrix, class VectorX, class VectorB>
  void DirichletBC<WorldVector>::finish(bool apply,
					Matrix& matrix,
                                        VectorX& solution,
					VectorB& rhs)
  {
    using Dune::Functions::interpolate;

    test_exit(initialized, "Boundary condition not initialized!");
    matrix.clearDirichletRows(dirichletNodes, apply);

    if (apply) {
      interpolate(matrix.getRowFeSpace(), wrapper(rhs.getVector()), values, dirichletNodes);
      interpolate(matrix.getColFeSpace(), wrapper(solution.getVector()), values, dirichletNodes);
    }
  }


} // end namespace AMDiS
