#include "StandardProblemIteration.hpp"

#include <dune/amdis/AdaptInfo.hpp>
#include <dune/amdis/Output.hpp>
#include <dune/amdis/ProblemStatBase.hpp>

namespace AMDiS
{
  int StandardProblemIteration::info = 10;


  ProblemStatBase& StandardProblemIteration::getProblem(int AMDIS_DBG_VAR( number ))
  {
    AMDIS_FUNCNAME_DBG("StandardProblemIteration::getProblem");
    test_exit_dbg(number == 0, "Problem number out of range!");
    return problem;
  }

  ProblemStatBase& StandardProblemIteration::getProblem(std::string AMDIS_DBG_VAR( name ))
  {
    AMDIS_FUNCNAME_DBG("StandardProblemIteration::getProblem");
    test_exit_dbg(name == problem.getName(), "Problem name does not match!\n");
    return problem;
  }

  void StandardProblemIteration::beginIteration(AdaptInfo& adaptInfo)
  {
    AMDIS_FUNCNAME("StandardProblemIteration::beginIteration()");

    msg("");
    msg("begin of iteration number: ", (adaptInfo.getSpaceIteration() + 1));
    msg("=============================");
  }


  Flag StandardProblemIteration::oneIteration(AdaptInfo& adaptInfo, Flag toDo)
  {
    Flag flag = buildAndAdapt(adaptInfo, toDo);

    if (toDo.isSet(SOLVE))
      problem.solve(adaptInfo, true, false);

    if (toDo.isSet(SOLVE_RHS))
      problem.solve(adaptInfo, true, false);

    if (toDo.isSet(ESTIMATE))
      problem.estimate(adaptInfo);

    return flag;
  }


  void StandardProblemIteration::endIteration(AdaptInfo& adaptInfo)
  {
    AMDIS_FUNCNAME("StandardProblemIteration::endIteration()");

    msg("");
    msg("end of iteration number: ", (adaptInfo.getSpaceIteration() + 1));
    msg("=============================");
  }


  Flag StandardProblemIteration::buildAndAdapt(AdaptInfo& adaptInfo, Flag toDo)
  {
    Flag flag = 0, markFlag = 0;

    if (toDo.isSet(MARK))
      markFlag = problem.markElements(adaptInfo);
    else
      markFlag = 3;

    if (toDo.isSet(BUILD))
      problem.buildBeforeRefine(adaptInfo, markFlag);

    // refine
    if (toDo.isSet(ADAPT) && markFlag.isSet(MESH_REFINED))
      flag = problem.refineMesh(adaptInfo);

    if (toDo.isSet(BUILD))
      problem.buildBeforeCoarsen(adaptInfo, markFlag);

    // coarsen
    if (toDo.isSet(ADAPT) && markFlag.isSet(MESH_COARSENED))
      flag |= problem.coarsenMesh(adaptInfo);

    if (toDo.isSet(BUILD))
      problem.buildAfterCoarsen(adaptInfo, markFlag, true, true);

    if (toDo.isSet(BUILD_RHS))
      problem.buildAfterCoarsen(adaptInfo, markFlag, false, true);

    return flag;
  }

  /// Returns the name of the problem.
  std::string StandardProblemIteration::getName() const
  {
    return problem.getName();
  }

} // end namespace AMDiS
