#pragma once

#include <chrono>

namespace AMDiS
{
  /// Helper class to measure execution times
  class Timer
  {
    using Clock     = std::chrono::high_resolution_clock;
    using TimePoint = std::chrono::time_point<Clock>;
    using fsec      = std::chrono::duration<double>;
    
  public:
    /// initializes the timer with current time
    Timer()
      : t0(Clock::now())
    {}

    /// resets the timer to current time
    void reset()
    {
      t0 = Clock::now();
    }

    /// returns the elapsed time (from construction or last reset) to now in seconds
    double elapsed() const
    {
      auto t1 = Clock::now();
      fsec fs = t1 - t0;
      return fs.count();
    }

  private:
    /// begin value for sequentiell measurement
    TimePoint t0;
  };

}  // end namespace AMDiS
