#pragma once

#include <type_traits>

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>

namespace AMDiS 
{
  namespace tag 
  {
    // some tags representing traits classes or categories
    struct none {};
    struct scalar {};
    struct vector {};
    struct matrix {};
    struct unknown {};

  } // end namespace tag

  /// Category of type T, e.g. scalar, vector matrix, specified by a tag
  template <class T, class = void>
  struct ValueCategory
  {
    using type = tag::unknown;
  };

  template <class T>
  using ValueCategory_t = typename ValueCategory<T>::type;
  
  template <class T>
  struct ValueCategory<T, std::enable_if_t< std::is_arithmetic<T>::value >>
  {
    using type = tag::scalar;
  };
  
  template <class K, int SIZE>
  struct ValueCategory< Dune::FieldVector<K, SIZE> >
  {
    using type = tag::vector;
  };

  template <class K>
  struct ValueCategory< Dune::FieldVector<K, 1> >
  {
    using type = tag::scalar;
  };

  template <class K, int ROWS, int COLS>
  struct ValueCategory< Dune::FieldMatrix<K, ROWS, COLS> >
  {
    using type = tag::matrix;
  };

  template <class K>
  struct ValueCategory< Dune::FieldMatrix<K, 1, 1> >
  {
    using type = tag::scalar;
  };
  
} // end namespace AMDiS
