#pragma once

#include "itl/block_diagonal.hpp"

#include <dune/amdis/CreatorMap.hpp>
#include <dune/amdis/linear_algebra/mtl/ITL_Preconditioner.hpp>

namespace AMDiS
{

  template <class SubMatrix, size_t _N, size_t _M, class Vector>
  struct DefaultCreators<PreconditionerInterface<BlockMTLMatrix<SubMatrix, _N, _M>, Vector>>
  {
    using Matrix = BlockMTLMatrix<SubMatrix, _N, _M>;
    using PreconBase = PreconditionerInterface<Matrix, Vector>;
    
    template <template<class> class ITLPrecon>
    using PreconCreator 
      = typename Preconditioner<Matrix, Vector, ITLPrecon<Matrix>>::Creator;
    
    using Map = CreatorMap<PreconBase>;
    
    static void init()
    { 
      auto pc_diag = new PreconCreator<DiagonalPreconditioner>;
      Map::addCreator("diag", pc_diag);
      Map::addCreator("jacobi", pc_diag);
      
      auto pc_id = new PreconCreator<IdentityPreconditioner>;
      Map::addCreator("identity", pc_id);
      Map::addCreator("no", pc_id);
      
      Map::addCreator("default", pc_id);
    }
  };

} // end namespace AMDiS
