#pragma once

#include <dune/amdis/linear_algebra/mtl/BlockMTLVector.hpp>
#include <dune/amdis/linear_algebra/mtl/BlockMTLMatrix.hpp>

#include <dune/amdis/linear_algebra/mtl/SystemVector.hpp>
#include <dune/amdis/linear_algebra/mtl/SystemMatrix.hpp>

#include <dune/amdis/linear_algebra/mtl/Mapper.hpp>

namespace AMDiS 
{
  // requires MatrixType to have an inserter
  template <class Value, size_t _N, size_t _M, class TargetType>
  void copy(BlockMTLMatrix<Value, _N, _M> const& source, TargetType& target)
  {
    using Mapper = std::conditional_t<(_N == _M), BlockMapper, RectangularMapper>;
    using value_type = typename TargetType::value_type;
    using BaseInserter = mtl::mat::inserter<TargetType, mtl::operations::update_plus<value_type>>;
    using MappedInserter = typename Mapper::template Inserter<BaseInserter>::type;
      
    size_t nnz = 0;
    for (size_t rb = 0; rb < _N; ++rb)
      for (size_t cb = 0; cb < _M; ++cb)
        nnz += source[rb][cb].nnz();	  
      
    {
      target.change_dim(num_rows(source), num_cols(source));
      set_to_zero(target);
      
      Mapper mapper(source);
      MappedInserter ins(target, mapper, int(1.2 * nnz / target.dim1()));
      
      for (size_t rb = 0; rb < _N; ++rb) {
        mapper.setRow(rb);
        for (size_t cb = 0; cb < _M; ++cb) {
	  mapper.setCol(cb);
          ins << source[rb][cb];
        }
      }
    }
  }
  
  template <class FeSpaces, class Value, class TargetType>
  void copy(SystemMatrix<FeSpaces, Value> const& source, TargetType& target)
  {
    copy(source.getMatrix(), target);
  }
  
} // end namespace AMDiS