#include <dune/amdis/AMDiS.hpp>
#include <dune/amdis/ProblemStat.hpp>
#include <dune/amdis/common/Literals.hpp>

#ifndef DIM
#define DIM 2
#endif
#ifndef DOW
#define DOW 2
#endif

using namespace AMDiS;

namespace {
  
  using TestParam   = ProblemStatTraits<DIM, DOW, 1>;
  using TestProblem = ProblemStat<TestParam>;
  using Op = TestProblem::OperatorType;

  class AMDiSTester
  {
  public:
    // create a problemStat
    AMDiSTester(std::string name)
      : prob(make_shared<TestProblem>(name))
    {
      prob->initialize(INIT_ALL);
    }
    
  
    void test1() // AddOperators
    {
      auto const& u = prob->getSolution<0>();
      
      // define operators
      Op op1;
      op1.addZOT( 1.0 ); 
      op1.addZOT( constant(1.0) );
      
      // value of DOFVector at QPs
      op1.addZOT( valueOf(u) );
      op1.addZOT( valueOfFunc(u, [](auto const& U) { return U; }) );
      op1.addZOT( valueOfFunc(u, [](auto const& U) { return U; }, 1) );
      
      // partial derivative of DOFVector
      op1.addZOT( derivativeOf(u, 0) );
      op1.addZOT( derivativeOf(u, 1) );
      
  //     op1.addFOT( gradientOf(u), GRD_PSI );
      
      // evaluate functor at coordinates
      op1.addZOT( eval([](auto const& x) { return x[0]; }) );
      op1.addZOT( [](auto const& x) { return 1.0; } );
      
      // add operator to system
      prob->addMatrixOperator(op1, 0, 0);
      prob->addVectorOperator(op1, 0);
      
      // define operators with factor argument
      Op op2;
      op2.addZOT( valueOf(u, 1.0) );
      
      op2.addZOT( derivativeOf(u, 0, 1.0) );
      op2.addZOT( derivativeOf(u, 1, 1.0) );
      
      // add operator to system, with optional factor as pointer
      double* factor = new double(1.0);
      prob->addMatrixOperator(op2, 0, 0, factor);
      prob->addVectorOperator(op2, 0, factor);
    }
  
  
    void test2() // MatrixVector 
    {
      auto const& u = prob->getSolution<0>();

      using GradOp = std::decay_t< decltype(gradientOf(u)) >;
      using DerivOp = std::decay_t< decltype(derivativeOf(u,0)) >;
      
      using Jacobian = typename GradOp::value_type;
      using Derivative = typename DerivOp::value_type;
      
      Jacobian v, w;
      
      double erg = v[0].dot(w[0]);
    }

  protected:
    shared_ptr<TestProblem> prob;
  };
  
} // end namespace


int main(int argc, char* argv[]) 
{  
  AMDiS::init(argc, argv);
    
  auto tester = make_shared<AMDiSTester>("test");
  tester->test1();
  tester->test2();
      
  AMDiS::finalize();
  return 0;
}